import sys
import time
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.alert import Alert
from selenium.common.exceptions import WebDriverException, TimeoutException

# コマンドライン引数をデバッグ出力する関数
def debug_print_arguments(args):
    print("Arguments received:", args)

# アクセスするURLをデバッグ出力する関数
def debug_print_url(url):
    print("URL being accessed:", url)

# WebDriverを初期化する関数
def initialize_webdriver(webdriver_path, browser_width, headless=True):
    chrome_options = Options()
    if headless:
        chrome_options.add_argument("--headless")  # ヘッドレスモード（GUIなし）で実行
        chrome_options.add_argument(f"--window-size={browser_width}x2160")  # 幅を指定し、高さを2160に設定
    else:
        chrome_options.add_argument("--start-maximized")  # 通常モードで最大化するオプションを追加

    chrome_options.add_argument("--ignore-certificate-errors")  # 証明書エラーを無視
    chrome_options.add_argument("--allow-running-insecure-content")  # 安全でないコンテンツの実行を許可
    service = Service(webdriver_path)
    return webdriver.Chrome(service=service, options=chrome_options)

# Basic認証の処理を行う関数
def handle_basic_auth(driver, basic_auth_id, basic_auth_pass):
    try:
        # 認証ダイアログが表示されるまで最大5秒待機
        WebDriverWait(driver, 5).until(EC.alert_is_present())
        alert = Alert(driver)
        alert.send_keys(f"{basic_auth_id}\n{basic_auth_pass}")  # IDとパスワードを入力
        alert.accept()  # 認証を実行
    except TimeoutException:
        print("The authentication dialog was not displayed.")  # 認証ダイアログが表示されない場合

# ページ全体をスクロールして完全に読み込み、スクリーンショットを取得する関数
def capture_screenshot(driver, file_path, browser_width):
    # 初期ウィンドウサイズを設定（高さは仮で2160）
    driver.set_window_size(browser_width, 2160)

    # ページ全体の読み込み完了を待機
    WebDriverWait(driver, 10).until(
        lambda d: d.execute_script("return document.readyState") == "complete"
    )

    # 現在のページの高さを取得（bodyとhtml要素の最大値を使用）
    last_height = driver.execute_script("return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight)")

    while True:
        # ページの最下部までスクロール
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        time.sleep(2)  # ページ描画を待機

        # 新しいページの高さを取得
        new_height = driver.execute_script("return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight)")

        if new_height == last_height:  # 高さが変わらなければ終了
            break
        last_height = new_height

    # 最終的なページの高さを取得し、300px 余裕を持たせてリサイズ
    final_height = driver.execute_script("return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight)") + 300
    driver.set_window_size(browser_width, final_height)  # ページ全体の高さにウィンドウを設定

    # 念のため再度最下部へスクロール（確実に表示させる）
    driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
    time.sleep(1)

    # スクリーンショットを保存
    driver.save_screenshot(file_path)  # スクリーンショットを保存

try:
    debug_print_arguments(sys.argv)  # コマンドライン引数をデバッグ出力
    
    # コマンドライン引数を取得
    url = sys.argv[1]
    file_path = sys.argv[2]
    webdriver_path = sys.argv[3]
    browser_width = int(sys.argv[4])
    
    # Basic認証情報の取得（オプション）
    basic_auth_id = sys.argv[5].encode('shift-jis').decode('utf-8') if len(sys.argv) > 5 else None
    basic_auth_pass = sys.argv[6].encode('shift-jis').decode('utf-8') if len(sys.argv) > 6 else None
    
    # URLが「http://」または「https://」で始まっていなければ「https://」を追加
    if not url.startswith("http://") and not url.startswith("https://"):
        url = "https://" + url
    debug_print_url(url)

    # Basic認証情報がある場合、URLに組み込む
    if basic_auth_id and basic_auth_pass:
        protocol, host = url.split("://")
        url = f"{protocol}://{basic_auth_id}:{basic_auth_pass}@{host}"
    debug_print_url(url)
    
    # WebDriverを初期化
    driver = initialize_webdriver(webdriver_path, browser_width)
    
    try:
        driver.get(url)  # 指定URLにアクセス
        
        # Basic認証を処理
        if basic_auth_id and basic_auth_pass:
            handle_basic_auth(driver, basic_auth_id, basic_auth_pass)
        
        # ページの読み込み完了を待機
        WebDriverWait(driver, 5).until(EC.presence_of_element_located((By.TAG_NAME, 'body')))
        
        # スクリーンショットをキャプチャ
        capture_screenshot(driver, file_path, browser_width)
        print(file_path)  # 保存先を出力
        
    except (WebDriverException, TimeoutException) as e:
        print(f"Err:An error has occurred: {str(e)}")  # エラーメッセージを出力
    finally:
        driver.quit()  # WebDriverを閉じる

# コマンドライン引数の不足エラー処理
except IndexError:
    print("Err:how to use: python script.py <url> <file_path> <webdriver_path> <browser_width> [<basic_auth_id> <basic_auth_pass>]")

# ブラウザ幅の数値変換エラー処理
except ValueError:
    print("Err:browser_width must be an integer.")

# その他の予期しないエラー処理
except Exception as e:
    print(f"Err:An Unexpected Error has occurred: {str(e)}")
